#!/usr/bin/ruby
#  encoding: UTF-8
#
# Check Barman Backup Status
# ===
#
# DESCRIPTION:
# This plugin checks the status and age of Barman backups for each configured server
# Do not run this script directly.  It needs to be called via check-barman-backup-status.rb
# because the barman gem requires sudo access to work properly and we have to run as
# the sensu user, so we use a sudo wrapper in check-barman-backup-status.rb
#
# PLATFORMS:
#   Linux, BSD, Solaris
#
# DEPENDENCIES:
#   Sensu Plugins run as the sensu user and barman needs to run with sudo or barman user
#   gem: sensu-plugin
#   gem: sudo
#   gem: rbarman
#
# LICENSE:
# Copyright 2016 Scott Brimhall <scottbrimhall@gmail.com>
#
# Released under the same terms as Sensu (the MIT license); see LICENSE
# for details.

require 'sensu-plugin/check/cli'
require 'time'

# main plugin class
class CheckBarmanBackupStatus < Sensu::Plugin::Check::CLI
  option :min_backups,
         description: 'Minimum number of backups that should exist before alerting',
         short:       '-m',
         long:        '--min-backups',
         required:    true,
         default:     1

  option :max_age,
         description: 'The maximum age of backup before critical alert (in hours)',
         long:        '--max-age',
         required:    true,
         default:     25

  option :homedir,
         description: 'The home directory path for barman',
         long:        '--homedir PATH',
         required:    true,
         default:     '/var/lib/barman'

  def time_diff(start_time, end_time)
    diff = end_time.to_i - start_time.to_i
    hours = (diff / 60 / 60)
    hours
  end

  def run
    start_time = Time.now
    @crit = []

    servers = `sudo barman list-server | awk '{print $1}'`.split(/\n/).map(&:strip)

    servers.each do |server|
      @server_hash = {
        server.to_s => {
          'attrs'   => {},
          'status'  => {}
        }
      }

      # No YAML or other supported output to parse so we'll have to do it manually
      server_attrs = `sudo barman show-server #{server}`.split(/\n/).map(&:strip)
      server_attrs.each do |attr|
        key, value = attr.split(':').map(&:strip)
        @server_hash[server]['attrs'][key] = value
      end

      server_status = `sudo barman status #{server}`.split(/\n/).map(&:strip)
      server_status.each do |status|
        key, value = status.split(':').map(&:strip)
        @server_hash[server]['status'][key] = value
      end

      if @server_hash[server]['status']['Active'] == 'False'
        @crit << "Server #{server} is not in active status.  Status is: #{@server_hash[server]['status']['Active']}"
      end

      check_status = `sudo barman check #{server} | grep -v 'Server #{server}'`.split(/\n/).map(&:strip)
      check_status.each do |check|
        key, value = check.split(':').map(&:strip)
        unless value =~ /^OK/
          @crit << "Status check failed on configured server #{server}.  #{key}: #{value}"
        end
      end

      backups = `sudo barman list-backup #{server} | awk '{print $2}'`.split(/\n/).map(&:strip)

      if backups.count < config[:min_backups].to_i && backups.count == 0
        @crit << "Minimum backups not met for server #{server}.  Count is #{backups.count}"
        next
      elsif backups.count < config[:min_backups].to_i && backups.count > 0
        @crit << "Minimum backups not met for server #{server}.  Count is #{backups.count}"
      end
      puts "Number of backups for server #{server}: #{backups.count}"

      @latest = backups[0]

      backups.each do |backup|
        stupid_barman_cli = `sudo barman list-backup #{server} | grep #{backup} | awk '{print $4}'`.strip
        if stupid_barman_cli == 'STARTED'
          if backup == @latest
            @skip_latest = true
          end
          puts "Backup ID #{backup} for server #{server} is in progress. Skipping further checking on it"
          next
        elsif stupid_barman_cli == 'FAILED'
          @crit << "Backup ID #{backup} for server #{server} is in failure state: #{stupid_barman_cli}"
          @skip_error = true
          next
        end
      end

      if @skip_latest == true
        puts 'Latest backup is in progress. Skipping age check.'
      elsif @skip_error == true
        puts 'Latest backup is in error state.  Skipping age check.'
      else
        latest_time = Time.parse(`sudo barman show-backup #{server} #{@latest} | grep 'Begin time' | awk '{print $4,$5}'`.strip)

        backup_age = time_diff(latest_time, start_time).to_i

        if backup_age > config[:max_age].to_i
          @crit << "Maximum age exceeded for latest backup on server #{server}. Age is: #{backup_age} hour(s)"
        end
        puts "Age of latest backup for server #{server}: #{backup_age} hour(s)"
      end
    end

    # Assuming we got this far, exit cleanly
    if @crit.empty?
      ok 'Recent backups exist for each configured server. Each configured server is also active and in a good state.'
    else
      critical "critical: #{@crit}"
    end
  end
end
